package cs108;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;

public final class LZWConcreteEncoder implements LZWEncoder {
    private final Map<String, Integer> dict;
    private final int dictCapacity;

    public LZWConcreteEncoder(SortedSet<Character> alphabet, int dictCapacity) {
        if (! (dictCapacity >= alphabet.size()))
            throw new IllegalArgumentException("dict. capacity smaller than alphabet");

        Map<String, Integer> dict = new HashMap<>(dictCapacity);
        for (char c: alphabet)
            dict.put(String.valueOf(c), dict.size());

        this.dict = dict;
        this.dictCapacity = dictCapacity;
    }

    @Override
    public List<Integer> encode(String str) {
        List<Integer> r = new ArrayList<>();
        StringBuilder preB = new StringBuilder();
        for (int i = 0; i < str.length(); ++i) {
            String pre = preB.append(str.charAt(i)).toString();
            if (dict.containsKey(pre))
                continue;

            if (dict.size() < dictCapacity)
                dict.put(pre, dict.size());
            r.add(dict.get(pre.substring(0, pre.length() - 1)));
            preB.delete(0, preB.length() - 1);
        }
        if (preB.length() > 0)
            r.add(dict.get(preB.toString()));
        return r;
    }
}
